require_relative 'errors/input_error'
require_relative 'errors/unknown_error'

module AddressService
  class Address
    class << self

      # List addresses for a given resource type
      def list(resource_type)
        path = "/resources/#{resource_type}/"

        response = client.get path
        response.body
      end

      # Find address for resource
      def find(resource_type, resource_id)
        path = "/resources/#{resource_type}/#{resource_id}"

        response = client.get path
        case response.status
        when 200
          # TODO: validate that response body is correct format.
          return response.body
        when 404
          raise Errors::ResourceNotFound
        else
          raise Errors::UnknownError
        end 
      end

      # Create an address for a given resource type
      def create(resource_type, params)
        path = "/resources/#{resource_type}/"
        response = client.post path, params

        case response.status
        when 201
          # TODO: validate that response body is correct format.
          return response.body
        when 422
          raise Errors::InputError, response.body
        else

          raise Errors::UnknownError
        end 
      end

      # Update an address
      def update(resource_type, resource_id, params)
        path = "/resources/#{resource_type}/#{resource_id}"

        response = client.patch path, params
        case response.status
        when 200
          # TODO: validate that response body is correct format.
          return response.body
        when 404
          raise Errors::ResourceNotFound
        when 422
          raise Errors::InputError, response.body
        else
          raise Errors::UnknownError
        end 
      end

      # Create an address for a given resource type
      def destroy(resource_type, resource_idu)
        path = "/resources/#{resource_type}/"
        response = client.delete path

        case response.status
        when 200
          # TODO: validate that response body is correct format.
          return response.body
        when 404
          raise Errors::ResourceNotFound
        else
          raise Errors::UnknownError
        end 
      end

      private

      def client
        AddressService.client
      end
    end
  end
end
